==> 0.11.3.tar.gz: lib/src/descriptor_builders/standard_impl.dart <==
library ac_dart.descriptor.builder.standard.impl;

import 'package:atlassian_connect_host/atlassian_connect_host.dart';
import 'package:mojito/mojito.dart'
    show ClientId, DefaultRoutePath, DefaultRouter, Oauth2RouteNames;
import 'package:uri/uri.dart';
import 'package:option/option.dart';
import 'package:atlassian_connect_shelf/src/descriptor_builders/descriptor_builder.dart';
import 'package:atlassian_connect_shelf/src/descriptor_builders/standard.dart';
import 'package:quiver/iterables.dart';

abstract class StandardDescriptorBuilderImpl<M extends ModuleList,
        S extends Scope, A extends ConnectAddon<M, S>>
    implements StandardDescriptorBuilder<M, S, A> {
  final List<_WebhookSpec> _webhooks = [];
  final List<ClassifiedModule> _modules = [];
  final String addonKey;
  final Uri baseUrl;

  StandardDescriptorBuilderImpl(this.addonKey, this.baseUrl);

  void addWebhook(String event, String name, Map<String, Object> params) {
    _webhooks.add(new _WebhookSpec(event, name, params));
  }

  M buildModules(DefaultRouter rootRouter, Option<ClientId> oauthClientId) {
    return buildProductModules(
        buildCoreModules(rootRouter, oauthClientId), rootRouter);
  }

  M buildProductModules(
      Iterable<ClassifiedModule> coreModules, DefaultRouter rootRouter);

  Iterable<ClassifiedModule> buildCoreModules(
      DefaultRouter rootRouter, Option<ClientId> oauthClientId) {
    final webLinkModules = _modules.where((m) => m.module is BasicWebLink).map(
        (m) => new ClassifiedModule(
            m.type,
            m.module.change(
                url: getMandatoryAbsoluteRouteUrlTemplate(
                    rootRouter, m.module.name.value))));

    final webhooks = _webhooks.map((spec) => createWebhook(spec, rootRouter));
    final oauthConsumer = createOAuthConsumer(oauthClientId);

    return concat([
      webLinkModules,
      webhooks,
      [oauthConsumer]
    ]).where((m) => m != null);
  }

  Lifecycle buildLifecycle(DefaultRouter rootRouter) {
    return getAbsoluteRouteUrl(
            rootRouter, DescriptorBuilder.installationRouteName)
        .map((Uri callbackUrl) => new Lifecycle(installed: callbackUrl))
        .getOrElse(() => null);
  }

  ClassifiedModule createOAuthConsumer(Option<ClientId> oauthClientId) {
    return (oauthClientId is Some
        ? new ClassifiedModule(CoreModuleTypes.oauthConsumer,
            new OAuthConsumer(oauthClientId.get().identifier))
        : null);
  }

  Option<UriTemplate> getAbsoluteRouteUrlTemplate(
      DefaultRouter rootRouter, String routeName) {
    return rootRouter.findByName(routeName).map((DefaultRoutePath rp) {
      return (new UriTemplate((new UriBuilder.fromUri(baseUrl)
        ..path += rp.fullPath.toString()).toString()));
    });
  }

  Option<Uri> getAbsoluteRouteUrl(DefaultRouter rootRouter, String routeName) {
    return rootRouter.findByName(routeName).map((DefaultRoutePath rp) {
      final Uri path = rp.fullPath.expand({});
      return (new UriBuilder.fromUri(baseUrl)..path += path.toString()).build();
    });
  }

  Uri getMandatoryAbsoluteRouteUrl(DefaultRouter rootRouter, String routeName) {
    return getAbsoluteRouteUrl(rootRouter, routeName)
        .getOrElse(() => throw new StateError('missing route $routeName'));
  }

  UriTemplate getMandatoryAbsoluteRouteUrlTemplate(
      DefaultRouter rootRouter, String routeName) {
    return getAbsoluteRouteUrlTemplate(rootRouter, routeName)
        .getOrElse(() => throw new StateError('missing route $routeName'));
  }

//  Oauth2Provider _buildOauth2Provider(DefaultRouter rootRouter) {
//    if (_routeNames == null) {
//      return null;
//    }
//
//    final userGrantRoute =
//        _getMandatoryAbsoluteRouteUrl(rootRouter, _routeNames.userGrantRoute);
//    final authTokenRoute =
//        _getMandatoryAbsoluteRouteUrl(rootRouter, _routeNames.authTokenRoute);
//
//    return new Oauth2Provider(
//        authorizationUrl: userGrantRoute, tokenUrl: authTokenRoute);
//  }

  @override
  void addGeneralPage(String key, String name, BitbucketLocation location) {
    addPage(CoreModuleTypes.generalPage, key, name, location);
  }

  @override
  void addAdminPage(String key, String name, BitbucketLocation location) {
    addPage(CoreModuleTypes.adminPage, key, name, location);
  }

  @override
  void addPage(
      String moduleType, String key, String name, BitbucketLocation location) {
    addModule(
        moduleType,
        new Page(
            key: key,
            name: new I18nProperty(value: name),
            location: location.key));
  }

  void addModule(String moduleType, Module module) {
    _modules.add(new ClassifiedModule(moduleType, module));
  }
}

ClassifiedModule createWebhook(_WebhookSpec spec, DefaultRouter rootRouter) {
  final Uri url = rootRouter
      .findByName(spec.name)
      .map((DefaultRoutePath rp) => rp.fullPath.expand({}))
      .getOrElse(() => throw new StateError(
          "Doh, can't find webhook route with name ${spec.name}"));

  return new ClassifiedModule(CoreModuleTypes.webHook, spec.toWebhook(url));
}

class _WebhookSpec {
  final String event, name;
  final Map params;

  _WebhookSpec(this.event, this.name, this.params);

  Webhook toWebhook(Uri url) =>
      new Webhook(event: event, url: url, params: params);
}

==> 0.11.6.tar.gz: lib/src/descriptor_builders/standard_impl.dart <==
library ac_dart.descriptor.builder.standard.impl;

import 'package:atlassian_connect_host/atlassian_connect_host.dart';
import 'package:atlassian_connect_shelf/src/descriptor_builders/standard.dart';
import 'package:mojito/mojito.dart'
    show ClientId, DefaultRoutePath, DefaultRouter, Oauth2RouteNames;
import 'package:option/option.dart';
import 'package:quiver/iterables.dart';
import 'package:uri/uri.dart';

int _maxId = 0;

abstract class StandardDescriptorBuilderImpl<M extends ModuleList,
        S extends Scope, A extends ConnectAddon<M, S>>
    implements StandardDescriptorBuilder<M, S, A> {
  final List<_WebhookSpec> _webhooks = [];
  final List<ClassifiedModule> _modules = [];
  final String addonKey;
  final Uri baseUrl;
  final int _id;
  final bool useAbsoluteUrls;

  String get installationRouteName => '$_id:acdart.installation.route';
  String get configurationRouteName => '$_id:acdart.configuration.route';
  String get descriptorRouteName => '$_id:acdart.descriptor.route';

  StandardDescriptorBuilderImpl(this.addonKey, this.baseUrl,
      {this.useAbsoluteUrls: false})
      : _id = _maxId++;

  void addWebhook(String event, String name, Map<String, Object> params) {
    _webhooks.add(new _WebhookSpec(event, name, params));
  }

  M buildModules(DefaultRouter rootRouter, Option<ClientId> oauthClientId) {
    return buildProductModules(
        buildCoreModules(rootRouter, oauthClientId), rootRouter);
  }

  M buildProductModules(
      Iterable<ClassifiedModule> coreModules, DefaultRouter rootRouter);

  Iterable<ClassifiedModule> buildCoreModules(
      DefaultRouter rootRouter, Option<ClientId> oauthClientId) {
    final webLinkModules = _modules.where((m) => m.module is BasicWebLink).map(
        (m) => new ClassifiedModule(
            m.type,
            m.module.change(url: getMandatoryRouteUrlTemplate(
                rootRouter, m.module.name.value))));

    final webhooks = _webhooks.map((spec) => createWebhook(spec, rootRouter));
    final oauthConsumer = createOAuthConsumer(oauthClientId);

    return concat([
      webLinkModules,
      webhooks,
      [oauthConsumer]
    ]).where((m) => m != null);
  }

  Lifecycle buildLifecycle(DefaultRouter rootRouter) {
    return getRouteUrl(rootRouter, installationRouteName)
        .map((Uri callbackUrl) => new Lifecycle(installed: callbackUrl))
        .getOrElse(() => null);
  }

  ClassifiedModule createOAuthConsumer(Option<ClientId> oauthClientId) {
    return (oauthClientId is Some
        ? new ClassifiedModule(CoreModuleTypes.oauthConsumer,
            new OAuthConsumer(oauthClientId.get().identifier))
        : null);
  }

  Option<UriTemplate> getRouteUrlTemplate(
      DefaultRouter rootRouter, String routeName) {
    return rootRouter.findByName(routeName).map((DefaultRoutePath rp) {
      return (new UriTemplate(
          (_routeUrlBuilder..path += rp.fullPath.toString()).toString()));
    });
  }

  UriBuilder get _routeUrlBuilder =>
      useAbsoluteUrls ? new UriBuilder.fromUri(baseUrl) : new UriBuilder()
        ..path = '/';

  Option<Uri> getRouteUrl(DefaultRouter rootRouter, String routeName) {
    return rootRouter.findByName(routeName).map((DefaultRoutePath rp) {
      final Uri path = rp.fullPath.expand({});
      return (_routeUrlBuilder..path += path.toString()).build();
    });
  }

  Uri getMandatoryRouteUrl(DefaultRouter rootRouter, String routeName) {
    return getRouteUrl(rootRouter, routeName)
        .getOrElse(() => throw new StateError('missing route $routeName'));
  }

  UriTemplate getMandatoryRouteUrlTemplate(
      DefaultRouter rootRouter, String routeName) {
    return getRouteUrlTemplate(rootRouter, routeName)
        .getOrElse(() => throw new StateError('missing route $routeName'));
  }

//  Oauth2Provider _buildOauth2Provider(DefaultRouter rootRouter) {
//    if (_routeNames == null) {
//      return null;
//    }
//
//    final userGrantRoute =
//        _getMandatoryAbsoluteRouteUrl(rootRouter, _routeNames.userGrantRoute);
//    final authTokenRoute =
//        _getMandatoryAbsoluteRouteUrl(rootRouter, _routeNames.authTokenRoute);
//
//    return new Oauth2Provider(
//        authorizationUrl: userGrantRoute, tokenUrl: authTokenRoute);
//  }

  @override
  void addGeneralPage(String key, String name, BitbucketLocation location) {
    addPage(CoreModuleTypes.generalPage, key, name, location);
  }

  @override
  void addAdminPage(String key, String name, BitbucketLocation location) {
    addPage(CoreModuleTypes.adminPage, key, name, location);
  }

  @override
  void addPage(
      String moduleType, String key, String name, BitbucketLocation location) {
    addModule(
        moduleType,
        new Page(
            key: key,
            name: new I18nProperty(value: name),
            location: location.key));
  }

  void addModule(String moduleType, Module module) {
    _modules.add(new ClassifiedModule(moduleType, module));
  }
}

ClassifiedModule createWebhook(_WebhookSpec spec, DefaultRouter rootRouter) {
  final Uri url = rootRouter
      .findByName(spec.name)
      .map((DefaultRoutePath rp) => rp.fullPath.expand({}))
      .getOrElse(() => throw new StateError(
          "Doh, can't find webhook route with name ${spec.name}"));

  return new ClassifiedModule(CoreModuleTypes.webHook, spec.toWebhook(url));
}

class _WebhookSpec {
  final String event, name;
  final Map params;

  _WebhookSpec(this.event, this.name, this.params);

  Webhook toWebhook(Uri url) =>
      new Webhook(event: event, url: url, params: params);
}
